@@ -108,6 +108,9 @@ class DataManager(object):
         """ return a shuffled iterator (record can be edit in place) """
         return self.iterator(shuffle=True)
 
+    def record_shuffle(self):
+        random.shuffle(self.record_list)
+
     def split(self, num_or_ratio, groupID_callback=None, random_seed=123):
         """
         Split data by the given condition
