@@ -222,4 +222,35 @@ def boxes_painter(rgb_image, box_list, label_list=None, score_list=None, color_l
     # get image with box and index
     array_image_with_box = np.asarray(pil_image)
 
-    return array_image_with_box
\ No newline at end of file
+    return array_image_with_box
+
+def adjust_bbox(bbox, offset_list):
+    """
+    [top, right, bottom, left]
+    [A] -> [-A, -A, A, A]
+    [A, B] -> [-A, -B, A, B]
+    [A, B, C, D] -< [A, B, C, D]
+    """
+    assert is_box(bbox), f"bbox {bbox} is not a box."
+    if is_number(offset_list):
+        offset_list = [offset_list]
+    
+    if len(offset_list) == 1:
+        A = offset_list[0]
+        offset_list = [-A, -A, A, A]
+    elif len(offset_list) == 2:
+        A, B = offset_list
+        offset_list = [-A, -B, A, B]
+    elif len(offset_list) == 4:
+        pass
+    else:
+        raise RuntimeError(f"unknown value {offset_list}")
+    
+    new_box = [
+        bbox[0] + offset_list[0],
+        bbox[1] + offset_list[1],
+        bbox[2] + offset_list[2],
+        bbox[3] + offset_list[3]
+    ]
+    
+    return new_box
\ No newline at end of file
