@@ -16,6 +16,7 @@ from .utils import is_integer
 from .utils import OS_exists
 from .utils import time_reduce
 from .utils import glob_recursively
+from .utils import has_chinese_char
 
 from tqdm import tqdm
 from pathlib import Path
@@ -308,19 +309,23 @@ def extend_image_channel(input_image):
     else:
         raise TypeError('image type error')
 
+def cv_bgr_imwrite(bgr_image, image_save_path, para=None, with_suffix=None, factor=1):
+    rgb_image = cv_rgb_bgr_convert(bgr_image)
+    cv_rgb_imwrite(rgb_image, image_save_path, para, with_suffix, factor)
 
-
-def cv_rgb_imwrite(rgb_image, image_save_path, bgr=False, para=None, with_suffix=None, factor=1):
+def cv_rgb_imwrite(rgb_image, image_save_path, para=None, with_suffix=None, factor=1):
     """
     [cv2 save a rgb image]
     Args:
         rgb_image ([np.array]): [rgb image]
         image_save_path ([str/Path]): [image save path]
     """
-    if not bgr:
-        image = to_colorful_image(rgb_image)
-    else:
-        image = rgb_image
+    image = rgb_image
+    if rgb_image.ndim == 3:
+        if rgb_image.shape[2] == 3:
+            bgr_image = cv_rgb_bgr_convert(rgb_image)
+            image = bgr_image
+
     image_save_path = Path(image_save_path)
     image_save_path.parent.mkdir(parents=True, exist_ok=True)
 
@@ -339,8 +344,16 @@ def cv_rgb_imwrite(rgb_image, image_save_path, bgr=False, para=None, with_suffix
     if factor != 1:
         image = image_resize(image, factor=factor)
 
-    cv.imwrite(str(image_save_path), image, quality_para)
+    save_path = str(image_save_path)
+    if has_chinese_char(save_path):
+        cv2.imencode(suffix, image, quality_para)[1].tofile(save_path)
+    else:
+        cv.imwrite(save_path, image, quality_para)
 
+def jpeg_noise(image, para):
+    encoded_img = cv2.imencode('.jpg', image, [cv2.IMWRITE_JPEG_QUALITY, para])[1]
+    decoded_img = cv2.imdecode(encoded_img, cv2.IMREAD_COLOR)
+    return decoded_img
 
 def pil_rgb_imwrite(rgb_image, image_save_path):
     """
@@ -453,6 +466,7 @@ def plt_image_show(*image, window_name='image show', array_res=False, full_scree
         plt.close()
         return image
 
+PIS = plt_image_show
 
 def convert_plt_to_rgb_image(plt):
     # Convert a Matplotlib figure to a 3D numpy array with RGB channels and return it
@@ -728,7 +742,7 @@ def get_polygons_from_mask(mask):
 
 
 def draw_polygons(mask, polygons, color=None, thickness=5, fill=False):
-
+    mask = mask.copy()
     if color is None:
         color = [255, 255, 255]
     if np.array(polygons).ndim == 2:
@@ -740,6 +754,19 @@ def draw_polygons(mask, polygons, color=None, thickness=5, fill=False):
 
     return mask
 
+def draw_boxes(image, bboxes, color=None, thickness=5, fill=False):
+    image = image.copy()
+    if color is None:
+        color = [255, 255, 255]
+    if np.array(bboxes).ndim == 1:
+        bboxes = [bboxes]
+    if fill:
+        thickness = -1
+    for box in bboxes:
+        box = vvd_round(box)
+        image = cv2.rectangle(image, box[:2], box[2:4], color, thickness)
+    return image
+
 
 def get_xyxys_from_mask(mask):
     """ get xyxys for all isolated areas on fg-mask """
@@ -751,6 +778,19 @@ def get_xyxys_from_mask(mask):
     return xyxys
 
 
+def get_polar_ring(image, center, outer_radius, inner_radius=0, factor=1):
+    size_X = vvd_round(outer_radius)
+    size_Y = vvd_round(outer_radius * 2 * np.pi)
+    
+    dsize = [size_X, size_Y]
+    center = vvd_round(center)
+    ori_polar = cv2.warpPolar(image, dsize, center, size_X, 12)
+    band = ori_polar[:, vvd_round(inner_radius):, ...]
+    if factor != 1:
+        band = image_resize(band, factor=factor)
+    return band
+
+
 def apply_gimp_brightness_contrast(input_img, brightness = 0, contrast = 0):
     """
     Gimp style brightness and contrast adjusification
