@@ -14,7 +14,9 @@ from os.path import join as OS_join
 from os.path import exists as OS_exists
 from os.path import isdir as OS_isdir
 from os.path import dirname as OS_dirname
+import string
 from numpy.lib.function_base import iterable
+from collections.abc import Iterator, Iterable
 
 import pypinyin
 
@@ -34,6 +36,7 @@ import sys
 import cv2 as cv
 import logging
 import os
+import random
 import platform
 import hashlib
 import pickle
@@ -47,6 +50,7 @@ from collections import OrderedDict
 from functools import wraps
 from functools import reduce
 
+from func_timeout import func_set_timeout, FunctionTimedOut
 
 
 def get_list_from_list(data_list, call_back, absolutely=False):
@@ -302,8 +306,8 @@ def current_split_char():
 def save_file_path_check(save_file_path, overwrite=False, verbose=False):
     """
     检查要保存的文件路径
-    - 如果文件已经存在 ： 在文件名与扩展名之间加入当前时间作为后缀 避免覆盖之前的文件并给出提示
-    - 如文件不存在 ： 检查文件所在的文件夹目录
+    - 如果文件已经存在 : 在文件名与扩展名之间加入当前时间作为后缀 避免覆盖之前的文件并给出提示
+    - 如文件不存在 : 检查文件所在的文件夹目录
     返回检查后的文件路径
     """
     if is_path_obj(save_file_path):
@@ -328,6 +332,13 @@ def save_file_path_check(save_file_path, overwrite=False, verbose=False):
     return checked_save_file_path
 
 
+def has_chinese_char(input_str):
+    for char in input_str:
+        if ord(char) > 255:
+            return True
+    return False
+
+
 def encode_chinese_to_unicode(input_string, remove=False):
     '''
     将中文转换为 unicode #Uxxxx 形式
@@ -369,8 +380,8 @@ def get_file_hash_code(file):
 class MyEncoder(json.JSONEncoder):
     """
     自定义序列化方法，解决 TypeError - Object of type xxx is not JSON serializable 错误
-    使用方法：
-    在json.dump时加入到cls中即可，例如：
+    使用方法:
+    在json.dump时加入到cls中即可，例如:
     json.dumps(data, cls=MyEncoder) 
     """
 
@@ -419,18 +430,43 @@ class Loger_printer():
 
     def __init__(self, logger):
         self.logger = logger
+        self.logger_dict = {
+            logging.DEBUG: self.logger.debug,
+            logging.INFO: self.logger.info,
+            logging.WARNING: self.logger.warning,
+            logging.ERROR: self.logger.error,
+            logging.CRITICAL: self.logger.critical
+        }
 
-    def vvd_logging(self, *message):
+    def get_logger_func(self, level):
+        if level in self.logger_dict:
+            return self.logger_dict[level]
+        else:
+            raise RuntimeError(f"unknown level {level}, {self.logger_dict}")
+
+    def vvd_logging(self, *message, level=logging.INFO):
+        log_func = self.get_logger_func(level)
         for message_str in message:
             print(message_str)
-            self.logger.info(message_str)
+            log_func(message_str)
 
-    def vvd_logging_quiet(self, *message):
+    def vvd_logging_quiet(self, *message, level=logging.INFO):
+        log_func = self.get_logger_func(level)
         for message_str in message:
-            self.logger.info(message_str)
+            log_func(message_str)
 
 
-def log_init(log_path, quiet=False):
+def get_date_str():
+    str_time = time.localtime(time.time())
+    year = str_time.tm_year
+    mounth = str_time.tm_mon
+    day = str_time.tm_mday
+
+    date_str = f"{year}-{mounth}-{day}"
+    return date_str
+
+
+def log_init(log_path, quiet=False, level=logging.INFO):
     """
     initialize logging 
     save the logging object in `config.Parameters.Logging_Object`
@@ -441,6 +477,7 @@ def log_init(log_path, quiet=False):
 
     Loger_printer.vvd_logging('test')
     """
+    log_path= str(log_path)
     dir_name = os.path.dirname(log_path)
 
     dir_check(dir_name)
@@ -454,30 +491,35 @@ def log_init(log_path, quiet=False):
         # open log file as  mode of write
         open_type = 'w'
 
-    logging.basicConfig(
+    # basicConfig 无法解决中文乱码的问题，封存相关代码
+    # 据说 Python 3.9 之后会加入basicConfig() 的encoding和errors关键字配置
+    # logging.basicConfig(
+    #     # 日志级别,logging.DEBUG,logging.ERROR
+    #     level=level,
+    #     # 日志格式: 时间、   日志信息
+    #     format='%(asctime)s: %(message)s',
+    #     # 打印日志的时间
+    #     datefmt='%Y-%m-%d %H:%M:%S',
+    #     # 日志文件存放的目录（目录必须存在）及日志文件名
+    #     filename=log_file_path,
+    #     # 打开日志文件的方式
+    #     filemode=open_type
+    # )
+    # logging.StreamHandler()
 
-        # 日志级别,logging.DEBUG,logging.ERROR
-        level=logging.INFO,
-
-        # 日志格式: 时间、   日志信息
-        format='%(asctime)s: %(message)s',
-
-        # 打印日志的时间
-        datefmt='%Y-%m-%d %H:%M:%S',
-
-        # 日志文件存放的目录（目录必须存在）及日志文件名
-        filename=log_file_path,
-
-        # 打开日志文件的方式
-        filemode=open_type
-    )
-
-    logging.StreamHandler()
+    # create logger obj
+    logger = logging.getLogger()
+    # set log level
+    logger.setLevel(level)
+    # file handler 
+    handler = logging.FileHandler(log_file_path, mode=open_type, encoding='utf-8')
+    handler.setFormatter(logging.Formatter("%(asctime)s-%(name)s-%(levelname)s: %(message)s"))
+    logger.addHandler(handler)
 
     if quiet:
-        return Loger_printer(logging).vvd_logging_quiet
+        return Loger_printer(logger).vvd_logging_quiet
     else:
-        return Loger_printer(logging).vvd_logging
+        return Loger_printer(logger).vvd_logging
 
 
 def dir_exists(dir_path):
@@ -867,28 +909,33 @@ def get_segments(data):
 def try_exc_handler(try_func, exc_func, developer_mode=False):
     except_result = try_result = None
 
+    def exception_handler(e, exc_func):
+        try:
+            ori_exception_info = list(e.args)
+            if len(ori_exception_info) == 0:
+                ori_exception_info.append('')
+            ori_exception_info[0] = ' ErrorMessage: ' + str(ori_exception_info[0])\
+                + '\n CrashFile: ' + str(e.__traceback__.tb_next.tb_frame.f_globals['__file__'])\
+                + '\n Line: ' + str(e.__traceback__.tb_next.tb_lineno)
+
+            e.args = tuple(ori_exception_info)
+            except_result = exc_func(e)
+        except Exception as ee:
+            print("!! exc_func failed.")
+            print(f"!! error message: {str(ee)}")
+            print("!! we can only return the previous exception info.")
+            except_result = str(e)
+        return except_result
+
     if developer_mode:
         try_result = try_func()
     else:
         try:
             try_result = try_func()
         except Exception as e:
-            try:
-                ori_exception_info = list(e.args)
-                if len(ori_exception_info) == 0:
-                    ori_exception_info.append('')
-                ori_exception_info[0] = ' ErrorMessage: ' + str(ori_exception_info[0])\
-                    + '\n CrashFile: ' + str(e.__traceback__.tb_next.tb_frame.f_globals['__file__'])\
-                    + '\n Line: ' + str(e.__traceback__.tb_next.tb_lineno)
-
-                e.args = tuple(ori_exception_info)
-                except_result = exc_func(e)
-            except Exception as ee:
-                print("!! exc_func failed.")
-                print(f"!! error message: {str(ee)}")
-                print("!! we can only return the previous exception info.")
-                except_result = str(e)
-            return except_result
+            return exception_handler(e, exc_func)
+        except FunctionTimedOut as e:
+            return exception_handler(e, exc_func)
 
     return try_result
 
@@ -1029,6 +1076,12 @@ class ListOrderedDict(OrderedDict):
 
         return super().__delitem__(key)
 
+    def __iter__(self): 
+        index = 0
+        while index < len(self):
+            yield self[index]      # 使用 yield
+            index = index + 1
+
     def move_to_end(self, key, last=True):
         '''Move an existing element to the end (or beginning if last is false).
 
@@ -1154,7 +1207,7 @@ def cartesian_to_polar(point_xy, center_xy, reverse_y=True):
         raise RuntimeError(f'bad distance {r}')
     return r, theta
 
-def polar_to_cartesian(r, theta, center_xy, reverse_y=True):
+def polar_to_cartesian(r, theta, center_xy, reverse_y=False):
     center_x, center_y = center_xy
     if reverse_y:
         y = center_y - r * math.sin(theta)
@@ -1166,7 +1219,142 @@ def polar_to_cartesian(r, theta, center_xy, reverse_y=True):
     return point_xy
 
 
+
+class LogicOp:
+    @staticmethod
+    def EqualTo(value):
+        def func(input):
+            if input == value:
+                return True
+            else:
+                return False
+        return func
+
+    @staticmethod
+    def NotEqualTo(value):
+        def func(input):
+            if input != value:
+                return True
+            else:
+                return False
+        return func
+
+    @staticmethod
+    def LargerThan(value):
+        def func(input):
+            if input > value:
+                return True
+            else:
+                return False
+        return func
+
+    @staticmethod
+    def LessThan(value):
+        def func(input):
+            if input < value:
+                return True
+            else:
+                return False
+        return func
+
+    @staticmethod
+    def NotLessThan(value):
+        def func(input):
+            if input >= value:
+                return True
+            else:
+                return False
+        return func
+    
+    @staticmethod
+    def NotLargerThan(value):
+        def func(input):
+            if input <= value:
+                return True
+            else:
+                return False
+        return func
+    
+    @staticmethod
+    def In(value):
+        assert iterable(value)
+        def func(input):
+            if input in value:
+                return True
+            else:
+                return False
+        return func
+
+    @staticmethod
+    def NotIn(value):
+        assert iterable(value)
+        def func(input):
+            if input in value:
+                return False
+            else:
+                return True
+        return func
+    
+    @staticmethod
+    def Is(value):
+        def func(input):
+            if input is value:
+                return True
+            else:
+                return False
+        return func
+    
+    @staticmethod
+    def IsNot(value):
+        def func(input):
+            if input is not value:
+                return True
+            else:
+                return False
+        return func
+    
+    @staticmethod
+    def IsTrue():
+        def func(input):
+            if input:
+                return True
+            else:
+                return False
+        return func
+    
+    @staticmethod
+    def IsFalse():
+        def func(input):
+            if not input:
+                return True
+            else:
+                return False
+        return func
+
+
+def random_string(length, digits=True, letters=True, punctuation=True):
+    str_temp = ''
+    if digits:
+        str_temp = str_temp + string.digits
+    if letters:
+        str_temp = str_temp + string.ascii_letters
+    if punctuation:
+        str_temp = str_temp + string.punctuation
+    return ''.join(random.choice(str_temp) for _ in range(length))
+
+
 if __name__ == '__main__':
+    LOD = ListOrderedDict()
+    LOD['a'] = 'a content'
+    LOD['b'] = 'b content'
+    LOD['c'] = 'c content'
+    LOD['d'] = 'd content'
+    
+    for value in LOD:
+        print(value)
+    for value in LOD:
+        print(value)
+    
     print(cartesian_to_polar([-1, 1], [0,0]))
     print(chinese_str_to_pinyin('asdfd_（阿斯蒂芬）'))
     
